'FileDir.bas    Ray Thomas      February 2002

'A file / directory dialog / menu

DECLARE SUB GetCurrent ()       '*** Get the current drive and directory ***
DECLARE SUB FindDrives ()       '*** Find which drives are on the PC ***
DECLARE SUB SortArrays (SortArray$(), Low AS INTEGER, High AS INTEGER)    '*** Put the files and directories in order ***
DECLARE SUB DrawMenu ()         '*** Draw the fixed menu items ***
DECLARE SUB FillMenu ()         '*** Fill the variable menu items ***
DECLARE SUB DoWriteMenu ()      '*** Do File, Drive, Directory menu items ***
DECLARE SUB DoFileMenu ()       '*** Do the scrolling file menu ***
DECLARE SUB DoDirMenu ()        '*** Do the scrolling directory menu ***
DECLARE SUB ChkFile ()          '*** Check that a file exists ***
DECLARE SUB ChkDrv ()           '*** Check that a drive exists ***
DECLARE SUB ChkDir ()           '*** Check that a directory exists ***
DECLARE SUB ChngDir ()          '*** Change a directory ***

OPTION BASE 1

'*** Global variables ***

DIM SHARED FileNo AS INTEGER            'File stream number
DIM SHARED DirName(500) AS STRING       'Directory names
DIM SHARED FileName(500) AS STRING      'File names
DIM SHARED CurrDrive AS STRING          'Current drive
DIM SHARED CurrDir AS STRING            'Current directory
DIM SHARED CurrPath AS STRING           'Current path
DIM SHARED NumDirs AS INTEGER           'Number of directories
DIM SHARED NumFiles AS INTEGER          'Number of files
DIM SHARED TabNum AS INTEGER            'Keep a track of what part of the menu
DIM SHARED YMenuPosn AS INTEGER         'Top right Y position of menu
DIM SHARED XMenuPosn AS INTEGER         'Top right X position of menu
DIM SHARED EndMenu AS INTEGER           'User pressed ESC or file opened properly
DIM SHARED HiLiteFile AS INTEGER        'Currently highlighted file
DIM SHARED HiLiteDir AS INTEGER         'Currently highlighted directory
DIM SHARED Message AS STRING            'Current field being edited
DIM SHARED Drive AS STRING              'Drives available on the computer
DIM SHARED ProgErr AS INTEGER           'Program error code
DIM SHARED OrigDrive AS STRING          'Original drive
DIM SHARED OrigDir AS STRING            'Original directory
DIM SHARED OrigPath AS STRING           'Original path


'*** Local variables ****

DIM Num AS INTEGER              'General integer variable

'*** The DIR command may give different results on different platforms ***

TabNum = 1
YMenuPosn = 3
XMenuPosn = 3

'*** Find the drives on the computer ***

ON ERROR GOTO DriveErr
FOR Count = 65 TO 90
        ProgErr = 0
        Message$ = CHR$(Count) + ":\"
        FILES Message$
        IF ProgErr = 0 THEN Drive$ = Drive$ + CHR$(Count)
NEXT Count
ON ERROR GOTO OopsError

GetCurrent      '*** Get the current drive and directory ***

SortArrays FileName$(), 1, NumFiles
SortArrays DirName$(), 1, NumDirs
           
'*** Get the original info to reset on exit ***

OrigDrive$ = CurrDrive$
OrigDir$ = CurrDir$
OrigPath$ = CurrPath$

DrawMenu        '*** Draw the fixed menu items ***
FillMenu        '*** Fill the variable menu items ***

DO

        '*** Relocate the cursor to the current field (by TabNum) ***
       
        EndMenu = 0

        IF TabNum <= 3 THEN
                DoWriteMenu
        END IF

        IF TabNum = 4 THEN DoFileMenu
        IF TabNum = 5 THEN DoDirMenu

LOOP UNTIL EndMenu > 0  '***  1 = ESC was pressed, 2 = file found, 3 = file created ***

CLS

IF EndMenu = 1 THEN

        PRINT
        PRINT "   Menu exited, ESC pressed"
END IF

IF EndMenu = 2 THEN

        PRINT
        PRINT "   The file "; Message$; " was successfully found"
        PRINT
        PRINT "   You may now open it or whatever"
END IF

IF EndMenu = 3 THEN
        PRINT
        PRINT "   The file "; Message$; " was successfully created"
        PRINT
        PRINT "   and now deleted"
       
        KILL Message$
END IF

'*** reset the drive and directory to the original ***

SHELL OrigDrive$
CHDIR OrigDir$

CLOSE
END

DriveErr:
IF ERR = 68 THEN ProgErr = 1
RESUME NEXT

OopsError:
CLS
PRINT
PRINT "   The program has reported a fatal error and will now end"
PRINT
PRINT "   The Error Code reported was"; ERR
PRINT
PRINT "   Press any key to continue . . ."
DO
LOOP UNTIL INKEY$ <> ""
CLOSE
SHELL OrigDrive$
CHDIR OrigDir$
END

FileErr:
IF ERR > 0 THEN ProgErr = 1
RESUME NEXT

SUB ChkDir

'*** There are two ways into this sub, either from a typed ***
'*** directory or one chosen from the directory menu ***

'*** Check that the chosen or typed directory exists ***
'*** It should already be in the FileName$ array,
'*** but refresh the array in case it was created while the program was running ***

DIM OldDir AS STRING            'A copy of the current directory name
DIM OldDrv AS STRING            'A copy of the current drive letter
DIM DirPosn AS INTEGER          'Place holder for splitting splitting directory string

OldDir$ = CurrDir$      '*** Keep a copy of the directory in case of errors ***
OldDrv$ = CurrDrive$    '*** Keep a copy of the drive in case of errors ***


IF TabNum = 5 THEN

        GetCurrent
        SortArrays FileName$(), 1, NumFiles
        SortArrays DirName$(), 1, NumDirs

        FOR Count = 1 TO UBOUND(DirName$)
      
                IF UCASE$(DirName$(HiLiteDir)) = UCASE$(DirName$(Count)) THEN
       
                        IF DirName$(HiLiteDir) = ".." THEN
                                SHELL "CD .."
                        ELSE
                                IF RIGHT$(CurrDir$, 1) <> "\" THEN CurrDir$ = CurrDir$ + "\"
                                SHELL "CD " + CurrDir$ + DirName$(HiLiteDir)
                        END IF
                        EXIT FOR
                END IF
        NEXT Count
END IF

'*** Getting the directories from the typed in string is a bit more complicated

'*** If the string starts with a letter then a ":" then the drive needs changing too ***
'*** If the string sarts with a "\" then assume the ***
'*** directory list starts from the root of the drive ***
'*** If it doesn't then assume it starts from the current directory ***

ProgErr = 0

IF TabNum = 3 THEN
       
        ON ERROR GOTO FileErr

        IF RIGHT$(Message$, 1) = "\" THEN Message$ = LEFT$(Message$, LEN(Message$) - 1)
             
        IF LEFT$(Message$, 1) = "\" THEN
                CHDIR Message$
        ELSE
                IF MID$(Message$, 2, 2) = ":\" THEN
                        IF INSTR(UCASE$(Drive$), UCASE$(LEFT$(Message$, 1))) > 0 THEN
                                SHELL LEFT$(Message$, 2)
                                Message$ = MID$(Message$, 3)
                                CHDIR Message$
                        END IF
                ELSE
                        Message$ = CurrDir$ + "/" + Message$
                        CHDIR Message$
                END IF
       
        END IF

        '*** If possible make the typed directory ***

        IF ProgErr = 1 THEN

                ProgErr = 0
                MKDIR Message$

                IF ProgErr = 0 THEN
                              
                        PRINT " created. Press any key"'
                        DO
                        LOOP UNTIL INKEY$ <> ""
                END IF
        END IF

        IF ProgErr = 1 THEN
                SHELL OldDrv$ + ":"
                CHDIR OldDir$
        END IF
       
        ON ERROR GOTO OopsError

        '*** Redo the command line ***
       
        GetCurrent
        LOCATE YMenuPosn + 5, XMenuPosn + 15
        PRINT SPACE$(59);
        Message$ = CurrDir$
        LOCATE YMenuPosn + 5, XMenuPosn + 15
        PRINT Message$;

END IF

HiLiteFile = 1
HiLiteDir = 1

GetCurrent
SortArrays FileName$(), 1, NumFiles
SortArrays DirName$(), 1, NumDirs
DrawMenu
FillMenu

END SUB

SUB ChkDrv

IF INSTR(1, Drive$, UCASE$(LEFT$(Message$, 1))) >= 1 THEN

        Message$ = UCASE$(LEFT$(Message$, 1)) + ":"
        SHELL Message$
       
        GetCurrent      '*** Get the current drive and directory ***

        SortArrays FileName$(), 1, NumFiles
        SortArrays DirName$(), 1, NumDirs

        DrawMenu
        FillMenu

END IF

END SUB

SUB ChkFile

DIM Count AS INTEGER

'*** Check that the chosen or typed file exists ***
'*** It should already be in the FileName$ array,
'*** but refresh the array in case it was created while the program was running ***

ProgErr = 0

GetCurrent
SortArrays FileName$(), 1, NumFiles
SortArrays DirName$(), 1, NumDirs

IF Message$ > "" THEN

        FOR Count = 1 TO UBOUND(FileName$)
       
                IF UCASE$(Message$) = UCASE$(FileName$(Count)) THEN
                        EndMenu = 2
                        EXIT FOR
                END IF
        NEXT Count

        IF EndMenu = 0 THEN     '*** Check the file can be created ***
               
                ON ERROR GOTO FileErr
                FileNo = FREEFILE
                OPEN Message$ FOR OUTPUT ACCESS WRITE LOCK READ WRITE AS FileNo
                CLOSE
                IF ProgErr = 0 THEN EndMenu = 3

                '*** If there was a problem creating this file then ProgErr = 1 ***

                ON ERROR GOTO OopsError
        END IF
END IF

END SUB

SUB DoDirMenu

'*** Scrolling directory menu ***

DIM UserIn AS STRING            'User input
DIM ChngTab AS INTEGER          'Tracks if menu movement keys are pressed
STATIC FirstDir AS INTEGER      'The first directory to be displayed
DIM YDirPosn AS INTEGER         '

ChngTab = 0
IF HiLiteDir = 0 THEN HiLiteDir = 1
IF FirstDir = 0 THEN FirstDir = 1
XDirPosn = XMenuPosn + 59

LOCATE (YMenuPosn + 9) + HiLiteDir - FirstDir, XDirPosn
COLOR 0, 7
PRINT DirName$(HiLiteDir);
COLOR 7, 0

DO

        '*** Print the up / down arrows ***
      
        LOCATE YMenuPosn + 9, XMenuPosn + 71
        PRINT "  ";
        LOCATE YMenuPosn + 18, XMenuPosn + 71
        PRINT "  ";
      
        IF FirstDir > 10 THEN
                LOCATE YMenuPosn + 9, XMenuPosn + 71
                PRINT CHR$(24); CHR$(24)
        END IF
      
        IF FirstDir < NumDirs - 9 THEN
                LOCATE YMenuPosn + 18, XMenuPosn + 71
                PRINT CHR$(25); CHR$(25)
        END IF
       
        LOCATE (YMenuPosn + 9) + HiLiteDir - FirstDir, XDirPosn + LEN(DirName$(HiLiteDir))
       
        DO
                UserIn$ = INKEY$
        LOOP UNTIL UserIn$ <> ""

        '*** Process the key pressed ***

        SELECT CASE UserIn$
                       
                CASE CHR$(27)   '*** Esc pressed ***

                        EndMenu = 1

                CASE CHR$(13)   '*** Enter pressed ***

                        '*** Check the directory exists, if it does change it ***
                        '*** This should'nt be necessary but it may have ***
                        '*** been deleted while this program was running ***
      
                        ChkDir

                CASE CHR$(9)    '*** Tab pressed ***
                      
                        TabNum = 1
                        ChngTab = 1

                CASE CHR$(0) + CHR$(15) '*** Shift Tab pressed ***

                        TabNum = TabNum - 1
                        ChngTab = 1
              
                CASE CHR$(0) + CHR$(80) '*** Down arrow pressed ***

                        IF DirName$(HiLiteDir + 1) <> "" THEN HiLiteDir = HiLiteDir + 1

                CASE CHR$(0) + CHR$(72) '*** Up arrow pressed ***

                        IF HiLiteDir - 1 <> 0 THEN HiLiteDir = HiLiteDir - 1


                CASE CHR$(0) + CHR$(71) '*** Home pressed ***

                                HiLiteDir = 1
              
                CASE CHR$(0) + CHR$(79) '*** End pressed ***

                        HiLiteDir = NumDirs

                CASE CHR$(0) + CHR$(73) '*** Page Up pressed ***

                        IF HiLiteDir > 10 THEN
                                HiLiteDir = HiLiteDir - 10
                        ELSE
                                HiLiteDir = 1
                        END IF
              
                CASE CHR$(0) + CHR$(81) '*** Page Down pressed ***
      
                        IF HiLiteDir < NumDirs - 9 THEN
                                HiLiteDir = HiLiteDir + 10
                        ELSE
                                HiLiteDir = NumDirs
                        END IF
              
                CASE ELSE
                              
                        '*** For any other key press find the first directory ***
                        '*** starting with that key ***

                        FOR Count = 1 TO NumDirs
                                IF UCASE$(LEFT$(DirName$(Count), 1)) = UCASE$(UserIn$) THEN
                                        HiLiteDir = Count
                                        EXIT FOR
                                END IF
                        NEXT Count

        END SELECT

        '*** Clear the directory list area and reprint the new list ***
      
        XDirPosn = XMenuPosn + 59
        LOCATE YMenuPosn + 9, XDirPosn
        FOR Count = FirstDir TO FirstDir + 9
                PRINT SPACE$(LEN(DirName$(Count)));
                LOCATE CSRLIN + 1, XDirPosn
        NEXT Count
              
        IF HiLiteDir > FirstDir + 9 THEN
                DO
                        FirstDir = FirstDir + 10
                LOOP UNTIL HiLiteDir < FirstDir + 10
        END IF

        IF HiLiteDir < FirstDir THEN
                DO
                        FirstDir = FirstDir - 10
                LOOP UNTIL HiLiteDir >= FirstDir
        END IF
              
        XDirPosn = XMenuPosn + 59
        LOCATE YMenuPosn + 9, XDirPosn
        FOR Count = FirstDir TO FirstDir + 9
                PRINT DirName$(Count);
                LOCATE CSRLIN + 1, XDirPosn
        NEXT Count
       
        XDirPosn = XMenuPosn + 59
        LOCATE (YMenuPosn + 9) + HiLiteDir - FirstDir, XDirPosn
        COLOR 0, 7
        PRINT DirName$(HiLiteDir);
        COLOR 7, 0

LOOP UNTIL EndMenu > 0 OR ChngTab = 1

END SUB

SUB DoFileMenu

'*** Scrolling Dir menu ***

DIM UserIn AS STRING            'User input
DIM ChngTab AS INTEGER          'Tracks if menu movement keys are pressed
DIM NumCols AS INTEGER          'Total number of Dir columns
STATIC FirstFile AS INTEGER     'The first file to be displayed
DIM XFilePosn AS INTEGER        '

NumCols = FIX((NumFiles + 9) / 10)      '*** calculates the number of columns ***
ChngTab = 0
IF HiLiteFile = 0 THEN HiLiteFile = 1
IF HiLiteFile = 1 THEN FirstFile = 1
IF FirstFile = 0 THEN FirstFile = 1
XFilePosn = XMenuPosn + 2

LOCATE (YMenuPosn + 9) + ((HiLiteFile - FirstFile) MOD 10), XFilePosn + (FIX((HiLiteFile - FirstFile) / 10) * 14)
COLOR 0, 7
PRINT FileName$(HiLiteFile);
COLOR 7, 0

DO
       
        '*** Print the << and / or the >> ***
       
        LOCATE YMenuPosn + 7, XMenuPosn + 2
        PRINT "  "
        LOCATE YMenuPosn + 7, XMenuPosn + 54
        PRINT "  "
       
        IF FirstFile > 10 THEN
                LOCATE YMenuPosn + 7, XMenuPosn + 2
                PRINT "<<"
        END IF
       
        IF FirstFile < NumFiles - 40 THEN
                LOCATE YMenuPosn + 7, XMenuPosn + 54
                PRINT ">>"
        END IF
        
        IF HiLiteFile > 1 THEN LOCATE (YMenuPosn + 9) + ((HiLiteFile - FirstFile) MOD 10), XFilePosn + (FIX((HiLiteFile - FirstFile) / 10) * 14) + LEN(FileName$(HiLiteFile))
        IF HiLiteFile = 1 THEN LOCATE (YMenuPosn + 9), XFilePosn + LEN(FileName$(HiLiteFile))
       
        DO
                UserIn$ = INKEY$
        LOOP UNTIL UserIn$ <> ""

        '*** Clear the user entry file box ***

        LOCATE CSRLIN, POS(0) - LEN(FileName$(HiLiteFile))
        COLOR 7, 0
        PRINT FileName$(HiLiteFile);
        LOCATE YMenuPosn + 1, XFilePosn + 13
        PRINT SPACE$(58)

        '*** Process the key pressed ***

        SELECT CASE UserIn$

                CASE CHR$(27)   '*** Esc pressed ***

                        EndMenu = 1

                CASE CHR$(13)   '*** Enter pressed ***

                        '*** ChkFile shouldn't be necessary but the chosen file ***
                        '*** may have been deleted while this program was running ***

                        ChkFile
                        
                CASE CHR$(9)    '*** Tab pressed ***
                       
                        TabNum = TabNum + 1
                        ChngTab = 1

                CASE CHR$(0) + CHR$(15) '*** Shift Tab pressed ***

                        TabNum = TabNum - 1
                        ChngTab = 1
               
                CASE CHR$(0) + CHR$(80) '*** Down arrow pressed ***

                        IF FileName$(HiLiteFile + 1) <> "" THEN HiLiteFile = HiLiteFile + 1

                CASE CHR$(0) + CHR$(72) '*** Up arrow pressed ***

                        IF HiLiteFile - 1 <> 0 THEN HiLiteFile = HiLiteFile - 1

                CASE CHR$(0) + CHR$(75) '*** Left arrow pressed ***

                        IF HiLiteFile > 10 THEN HiLiteFile = HiLiteFile - 10

                CASE CHR$(0) + CHR$(77) '*** Right arrow pressed ***

                        IF HiLiteFile < (NumCols - 1) * 10 + 1 THEN HiLiteFile = HiLiteFile + 10
                        IF HiLiteFile > NumFiles THEN HiLiteFile = NumFiles

                CASE CHR$(0) + CHR$(71) '*** Home pressed ***

                                HiLiteFile = 1
               
                CASE CHR$(0) + CHR$(79) '*** End pressed ***

                        HiLiteFile = NumFiles

                CASE CHR$(0) + CHR$(73) '*** Page Up pressed ***

                        IF HiLiteFile > 40 THEN
                                HiLiteFile = HiLiteFile - 40
                        ELSE
                                HiLiteFile = 1
                        END IF
               
                CASE CHR$(0) + CHR$(81) '*** Page Down pressed ***
       
                        IF HiLiteFile < NumFiles - 40 THEN
                                HiLiteFile = HiLiteFile + 40
                        ELSE
                                HiLiteFile = NumFiles
                        END IF
               
                CASE ELSE
                               
                        '*** For any other key press find the first file ***
                        '*** starting with that key ***

                        FOR Count = 1 TO NumFiles
                                IF UCASE$(LEFT$(FileName$(Count), 1)) = UCASE$(UserIn$) THEN
                                        HiLiteFile = Count
                                        EXIT FOR
                                END IF
                        NEXT Count

        END SELECT

        '*** Reprint the user entry file box ***
       
        Message$ = FileName$(HiLiteFile)
        LOCATE YMenuPosn + 1, XFilePosn + 13
        PRINT Message$
       
        '*** Clear the file list area and reprint the new columns ***
       
        XFilePosn = XMenuPosn + 2
        LOCATE YMenuPosn + 9, XFilePosn
        FOR Count = FirstFile TO FirstFile + 39
                PRINT SPACE$(12);
                LOCATE CSRLIN + 1, XFilePosn
                IF Count MOD 10 = 0 THEN
                        XFilePosn = XFilePosn + 14
                        LOCATE YMenuPosn + 9, XFilePosn
                END IF
        NEXT Count
               
        IF HiLiteFile > FirstFile + 39 THEN
                DO
                        FirstFile = FirstFile + 10
                LOOP UNTIL HiLiteFile < FirstFile + 40
        END IF

        IF HiLiteFile < FirstFile THEN
                DO
                        FirstFile = FirstFile - 10
                LOOP UNTIL HiLiteFile >= FirstFile
        END IF
               
        XFilePosn = XMenuPosn + 2
        LOCATE YMenuPosn + 9, XFilePosn
        FOR Count = FirstFile TO FirstFile + 39
                PRINT FileName$(Count);
                LOCATE CSRLIN + 1, XFilePosn
                IF Count MOD 10 = 0 THEN
                        XFilePosn = XFilePosn + 14
                        LOCATE YMenuPosn + 9, XFilePosn
                END IF
        NEXT Count
        XFilePosn = XMenuPosn + 2
      
        IF HiLiteFile > 1 THEN LOCATE (YMenuPosn + 9) + ((HiLiteFile - FirstFile) MOD 10), XFilePosn + (FIX((HiLiteFile - FirstFile) / 10) * 14)
        IF HiLiteFile = 1 THEN LOCATE (YMenuPosn + 9), XFilePosn
        COLOR 0, 7
        PRINT FileName$(HiLiteFile);
        COLOR 7, 0
      
LOOP UNTIL EndMenu > 0 OR ChngTab = 1

END SUB

SUB DoWriteMenu

'*** Do the user input for the file, drive and directory ***

DIM UserIn AS STRING            'User input
DIM CsrPosn AS INTEGER          'Cursor position
DIM InsFlag AS INTEGER          'The state of the Insert key
DIM ChngTab AS INTEGER          'Tracks if menu movement keys are pressed
DIM OldMessage AS STRING        'A copy of the original message

ChngTab = 0

IF TabNum = 1 THEN
        IF HiLiteFile = 0 THEN
                Message$ = FileName$(1)
        ELSE
                Message$ = FileName$(HiLiteFile)
        END IF
END IF

IF TabNum = 2 THEN Message$ = CurrDrive$

IF TabNum = 3 THEN Message$ = CurrDir$

CsrPosn = LEN(Message$) + 1


DO
        LOCATE YMenuPosn + (TabNum * 2) - 1, XMenuPosn + 15
        PRINT Message$;
        CsrPosn = LEN(Message$) + 1
        IF InsFlag >= 0 THEN
                LOCATE , XMenuPosn + 14 + CsrPosn, 1, 1
        ELSE
                LOCATE , XMenuPosn + 14 + CsrPosn, 1, 1, 30

        END IF
       
       
        UserIn$ = ""
       
        DO
                UserIn$ = INKEY$
        LOOP UNTIL UserIn$ <> ""

        SELECT CASE UserIn$

                CASE CHR$(27)   '*** Esc pressed ***

                        EndMenu = 1

                CASE CHR$(8)    '*** Backspace pressed ***
                       
                        LOCATE , XMenuPosn + 15
                        PRINT SPACE$(LEN(Message$))
                        IF CsrPosn > 1 THEN
                                Message$ = LEFT$(Message$, CsrPosn - 2) + MID$(Message$, CsrPosn)
                                CsrPosn = CsrPosn - 1
                        END IF
               
                CASE CHR$(0) + CHR$(83) '*** Delete key pressed ***
                       
                        LOCATE , XMenuPosn + 15
                        PRINT SPACE$(LEN(Message$))
                        Message$ = LEFT$(Message$, CsrPosn - 1) + MID$(Message$, CsrPosn + 1)

                CASE CHR$(0) + CHR$(82) '*** Ins key pressed ***

                        InsFlag = NOT InsFlag   '*** Boolean toggle ***

                CASE CHR$(0) + CHR$(75) '*** Left arrow pressed ***
                       
                        IF CsrPosn > 1 THEN CsrPosn = CsrPosn - 1
               
                CASE CHR$(0) + CHR$(77) '*** Right arrow pressed ***
                       
                        IF CsrPosn <= LEN(Message$) THEN CsrPosn = CsrPosn + 1
               
                CASE CHR$(0) + CHR$(71) '*** Home pressed ***
                       
                        CsrPosn = 1
               
                CASE CHR$(0) + CHR$(79) '*** End pressed ***
                       
                        CsrPosn = LEN(Message$) + 1
               
                CASE CHR$(0) + CHR$(15), CHR$(0) + CHR$(72)     '*** Up arrow or SHIFT Tab pressed ***
               
                        LOCATE , XMenuPosn + 15
                        PRINT SPACE$(59);
                        LOCATE , XMenuPosn + 15
                        IF HiLiteFile = 0 THEN HiLiteFile = 1
                        IF TabNum = 1 THEN Message$ = FileName$(HiLiteFile)
                        IF TabNum = 2 THEN Message$ = CurrDrive$
                        IF TabNum = 3 THEN Message$ = CurrDir$
                        PRINT Message$
                        TabNum = TabNum - 1
                        IF TabNum = 0 THEN TabNum = 5
                        ChngTab = 1

                CASE CHR$(9), CHR$(0) + CHR$(80)     '*** Down arrow or Tab pressed ***
              
                        LOCATE , XMenuPosn + 15
                        PRINT SPACE$(59);
                        LOCATE , XMenuPosn + 15
                        IF HiLiteFile = 0 THEN HiLiteFile = 1
                        IF TabNum = 1 THEN Message$ = FileName$(HiLiteFile)
                        IF TabNum = 2 THEN Message$ = CurrDrive$
                        IF TabNum = 3 THEN Message$ = CurrDir$
                        PRINT Message$
                        TabNum = TabNum + 1
                        ChngTab = 1

                CASE ELSE       '*** Add / insert the character to messages ***
                       
                        IF LEN(Message$) < 59 THEN
                                IF LEN(UserIn$) = 1 AND UserIn$ > CHR$(34) THEN
                                        IF InsFlag = 0 THEN
                                                Message$ = LEFT$(Message$, CsrPosn - 1) + UserIn$ + MID$(Message$, CsrPosn)
                                        ELSE
                                                Message$ = LEFT$(Message$, CsrPosn - 1) + UserIn$ + MID$(Message$, CsrPosn + 1)
                                        END IF
                                        CsrPosn = CsrPosn + 1
                                END IF
                        END IF
        END SELECT

        IF UserIn$ = CHR$(13) THEN
               
                SELECT CASE TabNum

                        CASE 1  '*** Check the file exists, if it does then exit menu ***

                                ChkFile

                        CASE 2  '*** Check the drive exists, if it does then change the drive ***

                                ChkDrv

                        CASE 3
                                '*** Check the directory exists, if it does then change it

                                ChkDir

                END SELECT

        END IF

LOOP UNTIL EndMenu > 0 OR ChngTab = 1

END SUB

SUB DrawMenu

'*** Draw the fixed menu items ***

DIM FileCols AS INTEGER         'Number of columns in the file section
DIM Count AS INTEGER            'Loop counter

FileCols = NumFiles / 10
XFilePosn = 7
YFilePosn = 9
XDirPosn = 65
YDirPosn = 9

CLS

LOCATE YMenuPosn, XMenuPosn
PRINT CHR$(201);
PRINT STRING$(12, CHR$(205));
PRINT CHR$(209);
PRINT STRING$(60, CHR$(205));
PRINT CHR$(187)
LOCATE , XMenuPosn
PRINT CHR$(186);
LOCATE , XMenuPosn + 2
PRINT "File";
LOCATE , XMenuPosn + 13
PRINT CHR$(179);
LOCATE , XMenuPosn + 74
PRINT CHR$(186)
LOCATE , XMenuPosn
PRINT CHR$(199);
PRINT STRING$(12, CHR$(196));
PRINT CHR$(197);
PRINT STRING$(60, CHR$(196));
PRINT CHR$(182)
LOCATE , XMenuPosn
PRINT CHR$(186);
LOCATE , XMenuPosn + 2
PRINT "Drive";
LOCATE , XMenuPosn + 13
PRINT CHR$(179);
LOCATE , XMenuPosn + 74
PRINT CHR$(186)
LOCATE , XMenuPosn
PRINT CHR$(199);
PRINT STRING$(12, CHR$(196));
PRINT CHR$(197);
PRINT STRING$(60, CHR$(196));
PRINT CHR$(182)
LOCATE , XMenuPosn
PRINT CHR$(186);
LOCATE , XMenuPosn + 2
PRINT "Directory";
LOCATE , XMenuPosn + 13
PRINT CHR$(179);
LOCATE , XMenuPosn + 74
PRINT CHR$(186)
LOCATE , XMenuPosn
PRINT CHR$(199);
PRINT STRING$(12, CHR$(196));
PRINT CHR$(193);
PRINT STRING$(43, CHR$(196));
PRINT CHR$(194);
PRINT STRING$(16, CHR$(196));
PRINT CHR$(182)
LOCATE , XMenuPosn
PRINT CHR$(186);
LOCATE , XMenuPosn + 7
PRINT NumFiles; "files in "; CurrPath$;
LOCATE , XMenuPosn + 57
PRINT CHR$(179);
LOCATE , XMenuPosn + 58
PRINT NumDirs; "Directories";
LOCATE , XMenuPosn + 74
PRINT CHR$(186)
LOCATE , XMenuPosn
PRINT CHR$(199);
PRINT STRING$(56, CHR$(196));
PRINT CHR$(197);
PRINT STRING$(16, CHR$(196));
PRINT CHR$(182)

FOR Count = 1 TO 10
        LOCATE , XMenuPosn
        PRINT CHR$(186);
        LOCATE , XMenuPosn + 57
        PRINT CHR$(179);
        LOCATE , XMenuPosn + 74
        PRINT CHR$(186)
NEXT Count

LOCATE , XMenuPosn
PRINT CHR$(200);
PRINT STRING$(56, CHR$(205));
PRINT CHR$(207);
PRINT STRING$(16, CHR$(205));
PRINT CHR$(188)

LOCATE YMenuPosn + 21, XMenuPosn
PRINT "Use the TAB key to move between fields"
PRINT

END SUB

SUB FillMenu

'*** Fill in the variable menu items ***

DIM XFilePosn AS INTEGER        ' X Positon of file columns

XFilePosn = XMenuPosn + 2

'*** The user writable part of the menu ***

LOCATE YMenuPosn + 3, XMenuPosn + 15
PRINT CurrDrive$
LOCATE YMenuPosn + 5, XMenuPosn + 15
PRINT CurrDir$

'*** the scrolling file menu ***

LOCATE YMenuPosn + 9, XFilePosn
FOR Count = 1 TO 40
        PRINT FileName$(Count)
        LOCATE CSRLIN, XFilePosn
        IF Count MOD 10 = 0 THEN
                XFilePosn = XFilePosn + 14
                LOCATE YMenuPosn + 9, XFilePosn
        END IF
NEXT Count

'*** The scrolling directory menu ***

LOCATE YMenuPosn + 9, XMenuPosn + 59
FOR Count = 1 TO 10
        PRINT DirName$(Count)
        LOCATE CSRLIN, XMenuPosn + 59
NEXT Count

END SUB

SUB FindDrives

'*** Find what drives are present in the PC ***

ON ERROR GOTO DriveErr
FOR Count = 65 TO 90
        DrvErr = 0
        Drive$ = CHR$(Count) + ":\"
        FILES Drive$
        IF DrvErr = 0 THEN Drive$ = Drive$ + CHR$(Count)
NEXT Count

END SUB

SUB GetCurrent

'*** Get the files from the current drive and directory ***

DIM Num AS INTEGER      'General number variable
DIM FileInfo AS STRING  'Line from ZFileDir.tmp

'*** Clear the current contents of the file and directory arrays ***

ERASE FileName$, DirName$

'*** Open a temporary file to hold the DIR information ***
'*** Unfortunately, I can't find a way of getting ***
'*** the location of the temporary file folder ***

FileNo = FREEFILE

SHELL "dir > c:\ZFileDir.tmp"

OPEN "C:\ZFileDir.tmp" FOR INPUT ACCESS READ LOCK READ WRITE AS FileNo

DO
        LINE INPUT #FileNo, FileInfo$
        IF EOF(FileNo) THEN EXIT DO

LOOP UNTIL INSTR(FileInfo$, "Directory of ") > 1

Num = INSTR(FileInfo$, ":")
Num = Num - 1
CurrDrive$ = MID$(FileInfo$, Num, 2)
CurrPath$ = MID$(FileInfo$, Num)
Num = Num + 2
CurrDir$ = MID$(FileInfo$, Num)
NumDirs = 0
NumFiles = 0

DO
        LINE INPUT #FileNo, FileInfo$
        IF EOF(FileNo) THEN EXIT DO
        IF INSTR(FileInfo$, "file(s)") > 0 THEN EXIT DO
        IF INSTR(FileInfo$, "<DIR>") > 1 THEN
                NumDirs = NumDirs + 1
                Num = INSTR(FileInfo$, " ")
                Num = Num - 1
                DirName$(NumDirs) = LEFT$(FileInfo$, Num)
                IF DirName$(NumDirs) = "." THEN
                        DirName$(NumDirs) = ""
                        NumDirs = NumDirs - 1
                END IF
        ELSE
                IF LEN(FileInfo$) > 0 THEN
                        NumFiles = NumFiles + 1
                        Num = INSTR(FileInfo$, " ")
                        Num = Num - 1
                        FileName$(NumFiles) = LEFT$(FileInfo$, Num) + "."
                        FileName$(NumFiles) = FileName$(NumFiles) + MID$(FileInfo$, 10, 3)
                        IF UCASE$(FileName$(NumFiles)) = "ZFILEDIR.TMP" THEN
                                FileName$(NumFiles) = ""
                                NumFiles = NumFiles - 1
                        END IF
                END IF
        END IF


LOOP UNTIL INSTR(FileInfo$, "file(s)") > 1
CLOSE
KILL "C:\ZFileDir.tmp"

END SUB

SUB SortArrays (SortArray$(), Low AS INTEGER, High AS INTEGER)

'*** This is a QuickSort routine ***

DIM Lower AS INTEGER
DIM Higher AS INTEGER
DIM RandIndex AS INTEGER

'QuickSort works by picking a random "pivot" element in SortArray, then
'moving every element that is bigger to one side of the pivot, and every
'element that is smaller to the other side.  QuickSort is then called
'recursively with the two subdivisions created by the pivot.  Once the
'number of elements in a subdivision reaches two, the recursive calls end
'and the array is sorted.

IF Low < High THEN

' *** Only two elements in this subdivision ***
' *** Swap them if they are out of order, then end recursive calls: ***

        IF High - Low = 1 THEN
                IF SortArray$(Low) > SortArray$(High) THEN
                SWAP SortArray$(Low), SortArray$(High)
        END IF

        ELSE

                '*** Pick a pivot element at random, then move it to the end ***
       
                RandIndex = INT(RND * (High - Low + 1)) + Low
                SWAP SortArray$(High), SortArray$(RandIndex)
                Partition$ = SortArray$(High)
                DO
               
                        '*** Move in from both sides towards the pivot element ***
               
                        Lower = Low
                        Higher = High
                        DO WHILE (Lower < Higher) AND (SortArray$(Lower) <= Partition$)
                                Lower = Lower + 1
                        LOOP
               
                        DO WHILE (Higher > Lower) AND (SortArray$(Higher) >= Partition$)
                                Higher = Higher - 1
                        LOOP
               
                        '*** If pivot element not reached, it means that ***
                        '*** two elements on either side are out of order, ***
                        '*** so swap them ***
               
                        IF Lower < Higher THEN
                                SWAP SortArray$(Lower), SortArray$(Higher)
                        END IF
       
                LOOP WHILE Lower < Higher

                '***  Move the pivot element back to its proper place in the array ***
       
                SWAP SortArray$(Lower), SortArray$(High)
        
                '*** Recursively call the SortArray sub ***
                '*** Pass the smaller subdivision first to use less stack space ***
       
                IF (Lower - Low) < (High - Lower) THEN
                        SortArrays SortArray$(), Low, Lower - 1
                        SortArrays SortArray$(), Lower + 1, High
                ELSE
                        SortArrays SortArray$(), Lower + 1, High
                        SortArrays SortArray$(), Low, Lower - 1
                END IF
        END IF
END IF

END SUB

